#######################################################
# SPECIFIC risks of the TSPM example                  #
# 17NRM05 EMUE Project - Activity A1.2.3              #
# Author: Francesca Pennecchi (INRIM)                 #
# Revised by Stephen Ellison (LGC)                    #
# Date: 09/10/2020    		                      #
#######################################################

rm(list=ls())

# SPECIFIC RISKS for the consumer (Rcspec) and the producer (Rpspec)
# with a single upper acceptance limit AV and permitted upper tolerance limit TU 

# Posterior pdf arising when the prior pdf is lognormal and the likelihood is normal
post <- function(eta, obs, ux=1, meanlog=0, sdlog=0) {
        unpost <- function(eta, obs, ux=1, meanlog=0, sdlog=0){
	         exp(dlnorm(eta, meanlog=meanlog, sdlog=sdlog, log=TRUE) + dnorm(obs, mean=eta, sd=ux*obs, log=TRUE))}
	  unpost.int <- integrate(unpost, lower=0, upper=Inf, obs=obs, ux=ux, meanlog = meanlog, sdlog = sdlog)
        normconst <- unpost.int$value
        unpost(eta, obs, ux, meanlog, sdlog) / normconst
}
# Check the post function's integral (should be 1.0):
integrate(post,lower=0,upper=Inf,obs=TU,ux=um,meanlog=mu1,sdlog=sigma1)

# Input data for the quarries
mu1 <- -2.325   	# Prior location parameter for Q1
mu2 <- -2.031   	# Prior location parameter for Q2
mu3 <- -2.337   	# Prior location parameter for Q3
sigma1 <- 0.434 	# Prior scale parameter for Q1
sigma2 <- 0.279 	# Prior scale parameter for Q2
sigma3 <- 0.402 	# Prior scale parameter for Q3
um <- 0.07	        # Relative measurement uncertainty
TU <- 0.2		# Upper tolerance limit


####################################################
# Consumer's specific risk for each observed value in [0, TU]

# Settings for numerical integrations
stepsize   <- 0.001
obsvaluesc <- seq(0,TU,stepsize)			# Observed values in [0, TU]
Rcspec1    <- rep(0,length(obsvaluesc))
Rcspec2    <- rep(0,length(obsvaluesc))
Rcspec3    <- rep(0,length(obsvaluesc))
eta        <- seq(0,1,stepsize)

# Q1
for(i in 3:length(obsvaluesc))
{
  Rcspec1[i] <- integrate(post,lower=TU,upper=Inf,obs=obsvaluesc[i],ux=um,meanlog=mu1,sdlog=sigma1)$value
}
obsvalues.required <- c(TU, 0.19, 0.18, 0.1)
approx(obsvaluesc, Rcspec1, obsvalues.required)  # 3.689340e-01 1.426158e-01 2.965583e-02 2.359741e-48

# Q2
for(i in 3:length(obsvaluesc))
{
  Rcspec2[i] <- integrate(post,lower=TU,upper=Inf,obs=obsvaluesc[i],ux=um,meanlog=mu2,sdlog=sigma2)$value
}
obsvalues.required <- c(TU, 0.19, 0.18, 0.1)
approx(obsvaluesc, Rcspec2, obsvalues.required)  # 3.303912e-01 1.235546e-01 2.504430e-02 4.687528e-48

# Q3
for(i in 3:length(obsvaluesc))
{
  Rcspec3[i] <- integrate(post,lower=TU,upper=Inf,obs=obsvaluesc[i],ux=um,meanlog=mu3,sdlog=sigma3)$value
} 
obsvalues.required <- c(TU, 0.19, 0.18, 0.1)
approx(obsvaluesc, Rcspec3, obsvalues.required)  # 3.506794e-01 1.325021e-01 2.686339e-02 1.779272e-48


####################################################
# Producer's specific risk for each observed value in [0.2, 0.4]

# Settings for numerical integrations
stepsize   <- 0.001
obsvaluesp <- seq(TU,0.4,stepsize)			# Observed values in [TU, 0.4], where 0.4 is in the extreme right tail of the posterior pdf 
Rpspec1    <- rep(0,length(obsvaluesp))
Rpspec2    <- rep(0,length(obsvaluesp))
Rpspec3    <- rep(0,length(obsvaluesp))

# Q1
for(i in 1:length(obsvaluesp))
{
  Rpspec1[i] <- integrate(post,lower=0,upper=TU,obs=obsvaluesp[i],ux=um,meanlog=mu1,sdlog=sigma1)$value
}
obsvalues.required <- c(TU, 0.225, 0.25)
approx(obsvaluesp, Rpspec1, obsvalues.required)  # 0.631066030 0.113340356 0.007412513

# Q2
for(i in 1:length(obsvaluesp))
{
  Rpspec2[i] <- integrate(post,lower=0,upper=TU,obs=obsvaluesp[i],ux=um,meanlog=mu2,sdlog=sigma2)$value
}
obsvalues.required <- c(TU, 0.225, 0.25)
approx(obsvaluesp, Rpspec2, obsvalues.required)  # 0.6696088 0.1430952 0.0120262

# Q3
for(i in 1:length(obsvaluesp))
{
  Rpspec3[i] <- integrate(post,lower=0,upper=TU,obs=obsvaluesp[i],ux=um,meanlog=mu3,sdlog=sigma3)$value
} 
obsvalues.required <- c(TU, 0.225, 0.25)
approx(obsvaluesp, Rpspec3, obsvalues.required)  # 0.649320598 0.124398970 0.008791669


############################################################
# Plots
par(mfrow=c(1,2))

# Consumer's risks
plot(obsvaluesc[obsvaluesc>0.15],Rcspec1[obsvaluesc>0.15], xlim=c(0.15,0.20), ylim=c(0,0.35), col = "blue", type ="l", lwd=2, font.axis = 6, axes = FALSE, cex.axis=1.5, main = "", ylab = "", xlab = "")
axis(1, xaxp=c(0.15,0.20,10), font.axis = 6, cex.axis=1.3)
axis(2, yaxp=c(0,0.35,5), font.axis = 6, cex.axis=1.3)
mtext(family = "serif", cex = 1.5, text = expression( paste(italic(c)[m]/group("(",list(mg~m^-"\u0033"),")")) ),side = 1,line = 3)
mtext(family = "serif", cex = 1.5, text = expression("Risk value"),side = 2,line = 2.5)
title(family = "serif", main="Specific consumer's risks", xlab = " ", cex.lab = 1.8, font.main = 6, font.lab = 6, cex.main = 2)  
lines(obsvaluesc[obsvaluesc>0.15],Rcspec2[obsvaluesc>0.15],col='green')
lines(obsvaluesc[obsvaluesc>0.15],Rcspec3[obsvaluesc>0.15], col='red')

# Producer's risks
plot(obsvaluesp[obsvaluesp>0.17],Rpspec1[obsvaluesp>0.17], xlim=c(0.20,0.25), ylim=c(0,0.7), col = "blue", type ="l", lwd=2, font.axis = 6, axes = FALSE, cex.axis=1.5, main = "", ylab = "", xlab = "")
axis(1, xaxp=c(0.20,0.25,10), font.axis = 6, cex.axis=1.3)
axis(2, yaxp=c(0,0.70,5), font.axis = 6, cex.axis=1.3)
mtext(family = "serif", cex = 1.5, text = expression( paste(italic(c)[m]/group("(",list(mg~m^-"\u0033"),")")) ),side = 1,line = 3)
mtext(family = "serif", cex = 1.5, text = expression("Risk value"),side = 2,line = 2.5)
title(family = "serif", main="Specific producer's risks", xlab = " ", cex.lab = 1.8, font.main = 6, font.lab = 6, cex.main = 2)  
lines(obsvaluesp[obsvaluesp>0.17],Rpspec2[obsvaluesp>0.17],col='green')
lines(obsvaluesp[obsvaluesp>0.17],Rpspec3[obsvaluesp>0.17], col='red')


